/*
 * blanco Framework
 * Copyright (C) 2004-2006 IGA Tosiki
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 */
package blanco.xml.bind;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Logger;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import blanco.xml.bind.valueobject.BlancoXmlDocument;

/**
 * XMLJavaIuWFNg𐶐܂B
 * 
 * ̃NX XML/o[IuWFNg}bsO (X/O}bsO) blancoXmlBinding ̈ꕔłB
 * 
 * @author IGA Tosiki
 */
public class BlancoXmlUnmarshaller {
    /**
     * XMLJavaIuWFNg𐶐܂B
     * 
     * Ql: http://java.sun.com/webservices/docs/1.6/api/javax/xml/bind/
     * Unmarshaller.html
     * 
     * @param fileInput
     *            XMLt@CB
     * @return blancoXmlƂẴIuWFNgB
     */
    public BlancoXmlDocument unmarshal(final File fileInput) {
        if (fileInput.exists() == false) {
            throw new IllegalArgumentException("BlancoXmlUnmarshaller: t@C["
                    + fileInput.getAbsolutePath() + "]͌܂B");
        }
        if (fileInput.isDirectory()) {
            throw new IllegalArgumentException("BlancoXmlUnmarshaller: t@C["
                    + fileInput.getAbsolutePath() + "]̓fBNgłB");
        }
        if (fileInput.canRead() == false) {
            throw new IllegalArgumentException("BlancoXmlUnmarshaller: t@C["
                    + fileInput.getAbsolutePath() + "]͓ǂݍ݂ł܂B");
        }

        InputStream inStream = null;

        try {
            inStream = new BufferedInputStream(new FileInputStream(fileInput));

            return unmarshal(inStream);
        } catch (IOException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(
                    "BlancoXmlUnmarshaller: ɓo͗O܂B" + e.toString());
        } finally {
            if (inStream != null) {
                try {
                    inStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                    throw new IllegalArgumentException(
                            "BlancoXmlUnmarshaller: ɓo͗O܂B"
                                    + e.toString());
                }
            }
        }
    }

    /**
     * XMLJavaIuWFNg𐶐܂B
     * 
     * Ql: http://java.sun.com/webservices/docs/1.6/api/javax/xml/bind/
     * Unmarshaller.html
     * 
     * @param inStream
     *            XML̓Xg[B
     * @return blancoXmlƂẴIuWFNgB
     */
    public BlancoXmlDocument unmarshal(final InputStream inStream) {
        if (inStream == null) {
            throw new IllegalArgumentException(
                    "BlancoXmlUnmarshaller: ̓Xg[null^܂B");
        }

        final BlancoXmlUnmarshallerContentHandler handler = new BlancoXmlUnmarshallerContentHandler();

        try {
            // XMLp[T̐ݒRg[̂ŁAXMLReaderD悵ėpB
            XMLReader reader = null;
            try {
                reader = XMLReaderFactory.createXMLReader();
            } catch (SAXException e) {
                Logger.getLogger("blanco.xml.bind").fine(
                        "XMLReader̎擾Ɏs܂: " + e.toString());

                // ɂĂ(JDK 1.4.2ŋH?) XMLReader̎擾Ɏsꍇ܂B

                // XMLReaderFactory.createXMLReader() ĂяoɁA
                // org.xml.sax.SAXException: System property org.xml.sax.driver
                // not specified 邱ƂmFĂ܂B
                final SAXResult result = new SAXResult(handler);
                result.setHandler(handler);
                result.setLexicalHandler(handler);
                final TransformerFactory tf = TransformerFactory.newInstance();
                try {
                    final Transformer transformer = tf.newTransformer();
                    transformer.transform(new StreamSource(inStream), result);
                    return handler.getDocument();

                } catch (TransformerException e2) {
                    throw new IllegalArgumentException(
                            "BlancoXmlUnmarshaller: XML͗O܂B"
                                    + e2.toString());
                }
            }

            // XMLReader𖳎Ɏ擾łꍇɂ́Aʂ܂B

            try {
                // ODTDǂݍ܂Ȃ邽߂̎B
                reader
                        .setFeature(
                                "http://apache.org/xml/features/nonvalidating/load-external-dtd",
                                false);
            } catch (SAXNotRecognizedException e) {
                Logger.getLogger("blanco.xml.bind").finest(
                        "ODTDǂݍ܂Ȃݒ̎{Ɏs܂: " + e.toString());
            }

            reader.setContentHandler(handler);
            // Rg邽߂ɃvpeBݒB
            reader.setProperty("http://xml.org/sax/properties/lexical-handler",
                    handler);
            reader.parse(new InputSource(inStream));
            return handler.getDocument();

        } catch (SAXException e) {
            // TODO ꂽ catch ubN
            e.printStackTrace();
            throw new IllegalArgumentException(
                    "BlancoXmlUnmarshaller: XML͗O܂B" + e.toString());
        } catch (IOException e) {
            // TODO ꂽ catch ubN
            e.printStackTrace();
            throw new IllegalArgumentException(
                    "BlancoXmlUnmarshaller: ɓo͗O܂B" + e.toString());
        } finally {
            if (inStream != null) {
                try {
                    inStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                    throw new IllegalArgumentException(
                            "BlancoXmlUnmarshaller: ɓo͗O܂B"
                                    + e.toString());
                }
            }
        }
    }
}
